<?php
/*
Plugin Name: 	First payment date for WooCommerce Subscriptions
Plugin URI: 	https://codection.com/first-payment-date-for-wooCommerce-subscriptions
Description: 	This plugin allows you to set a first payment date for a subscription, using WooCommerce Subscriptions. So you will be able to set a specific date (not a generic trial period) for each product
Version: 		0.5.3
Author: 		Codection
Author URI: 	https://codection.com
License:        GPL2
License URI:    https://www.gnu.org/licenses/gpl-2.0.html
Text Domain: 	first-payment-date-for-woocommerce-subscriptions
Domain Path: 	/languages
WC requires at least: 2.4.0
WC tested up to: 7.7
*/
add_action( 'init', function(){
    $fpdws = new FirstPaymentDateForWooCommerceSubscriptions();
	$fpdws->load_hooks();
}, 0 );

class FirstPaymentDateForWooCommerceSubscriptions{
	var $premium_url;

	function __construct(){
		$this->premium_url = 'https://import-wp.com/payment-schedule-for-woocommerce-subscriptions?utm_source=wp-free-dashboard&utm_medium=first-payment-date-for-woocommerce-subscriptions&utm_campaign=button-upgrade';
	}

	function load_hooks(){
		add_action( 'init', array( $this, 'load_textdomain' ) );
		add_filter( 'plugin_action_links', array( $this, 'add_action_link' ), PHP_INT_MAX, 2 );
		add_filter( 'woocommerce_subscriptions_product_trial_length', array( $this, 'woocommerce_subscriptions_product_trial_length' ), 10, 2 );
		add_filter( 'woocommerce_subscriptions_product_trial_period', array( $this, 'woocommerce_subscriptions_product_trial_period' ), 10, 2 );
		add_filter( 'woocommerce_add_to_cart_validation', array( $this, 'woocommerce_add_to_cart_validation' ), PHP_INT_MAX, 4 );
		add_action( 'woocommerce_product_options_general_product_data', array( $this, 'set_first_date_payment_simple_subscription' ) );
		add_action( 'woocommerce_process_product_meta', array( $this, 'update_first_date_payment_simple_subscription' ) );
		add_action( 'woocommerce_product_after_variable_attributes', array( $this, 'set_first_date_payment_variable_subscription' ), 10, 3 );
		add_action( 'woocommerce_save_product_variation', array( $this, 'update_first_date_payment_variable_subscription' ), 10, 2 );
		add_action( 'woocommerce_variable_product_bulk_edit_actions', array( $this, 'bulk_edit_actions' ), 10 );
		add_action( 'woocommerce_bulk_edit_variations', array( $this, 'bulk_edit_variations' ), 10, 4 );
		add_action( 'admin_footer', array( $this, 'footer_scripts' ) );
		add_filter( 'woocommerce_subscription_lengths', array( $this, 'change_woocommerce_subscription_lengths' ), 10, 2 );
		add_action( 'fpdws_after_simple_subscription_product_options', array( $this, 'show_payment_schedule_message_simple' ) );
		add_action( 'fpdws_after_variation_subscription_product_options', array( $this, 'show_payment_schedule_message_variation' ) );	
	}

	function load_textdomain(){
		load_plugin_textdomain( 'first-payment-date-for-woocommerce-subscriptions', false, basename( dirname( __FILE__ ) ) . '/languages' ); 
	}
	
	
	function add_action_link( $links, $file ) {
		$plugin_base = plugin_basename( __FILE__ );

		if ( $file == $plugin_base ) {
			if( !function_exists( 'is_plugin_active' ) )
				include_once ABSPATH . 'wp-admin/includes/plugin.php';

			if( is_plugin_active( 'payment-schedule-for-woocommerce-subscriptions/payment-schedule-for-woocommerce-subscriptions.php' ) )
				return $links;

            $premium_link = '<a href="' . $this->premium_url . '" style="color:#93003c; font-weight:bold;">' . esc_html__( 'Premium Version', 'first-payment-date-for-woocommerce-subscriptions' ) . '</a>';
            $links[] = $premium_link;
		}

		return $links;
	}

	function woocommerce_subscriptions_product_trial_length( $subscription_trial_length, $product ){
		if( empty( $product ) )
			return $subscription_trial_length;
	
		$custom_trial_length = $this->calculate_custom_trial_length( $product );
		return ( $custom_trial_length === false ) ? $subscription_trial_length : $custom_trial_length;
	}
	
	function woocommerce_subscriptions_product_trial_period( $subscription_trial_period, $product ){
		if( is_admin() || empty( $product ) )
			return $subscription_trial_period;
	
		$id = $product->get_id();
		if( $product->get_type() == 'variable-subscription' ){
			$variations = $product->get_children();
			$id = $variations[0];
		}
	
		$date = get_post_meta( $id, '_fpdws_date_first_payment', true );
	
		if( empty( $date ) || $date == 'none' )
			return $subscription_trial_period;
	
		return 'day';
	}

	function woocommerce_add_to_cart_validation( $passed, $product_id, $quantity, $variation_id = '' ) {
		$id = empty( $variation_id ) ? $product_id : $variation_id;

		$product = wc_get_product( $id );
		$custom_trial_length = $this->calculate_custom_trial_length( $product );
		if( $custom_trial_length === false ) 
			return $passed;
		
		$product->update_meta_data( '_subscription_trial_length', $custom_trial_length );
		$product->update_meta_data( '_subscription_trial_period', 'day' );
		$product->save();
		
		return $passed;
	}

	function calculate_custom_trial_length( $product ){
		$id = $product->get_id();
		if( $product->get_type() == 'variable-subscription' ){
			$variations = $product->get_children();
			$id = $variations[0];
		}
	
		$type = get_post_meta( $id, '_fpdws_date_first_payment_type', true );
		
		if( $type == 'none' ){
			return false;
		}

		$timezone = new DateTimeZone ( wp_timezone_string() );
		$earlier = new DateTime( 'now', $timezone );
	
		if( $type == 'fixed' || empty( $type ) ){
			$date = get_post_meta( $id, '_fpdws_date_first_payment', true );
	
			if( empty( $date ) )
				return false;
	
			try {
				$later = new DateTime( $date, $timezone );
				$later->modify( '+1 day' );
			} catch (Exception $e) {
				return false;
			}			
		}
		elseif( $type == 'calculated' ){
			$calculated = get_post_meta( $id, '_fpdws_date_first_payment_calculated', true );
	
			if( $calculated == 'first_day_next_month' )
				$later = new DateTime( 'first day of next month', $timezone );
			elseif( $calculated == 'first_day_next_year' )
				$later = new DateTime( ( date( 'Y' ) + 1 ) . '-01-01', $timezone );
		}

		if( isset( $original_subscription_trial_days ) && $original_subscription_trial_days > 0 ){
			$later->modify("+$original_subscription_trial_days days");
		}

		if( $earlier->getTimestamp() >= $later->getTimestamp() )
			return 0;
		
		return $later->diff($earlier)->format("%a");
	}

	function set_first_date_payment_simple_subscription() {
		global $post;
	
		$product = wc_get_product( $post->ID );
	
		if( $product->has_child() )
			return;
	
		echo '<div class="options_group show_if_subscription">';
	
		woocommerce_wp_select( array(
			'id'      => '_fpdws_date_first_payment_type',
			'label'   => __( 'First payment date type', 'first-payment-date-for-woocommerce-subscriptions' ),
			'options' => array( 
				'none' => __( 'Use standard trial period', 'first-payment-date-for-woocommerce-subscriptions' ),
				'fixed' => __( 'Fixed date', 'first-payment-date-for-woocommerce-subscriptions' ),
				'calculated' => __( 'Calculated day', 'first-payment-date-for-woocommerce-subscriptions' ),
			),
			'value'	  => get_post_meta( $product->get_id(), '_fpdws_date_first_payment_type', true ),
		) );
	
		woocommerce_wp_text_input(
			array(
				'id'          => '_fpdws_date_first_payment',
				'label'       => __( 'Fixed first payment date', 'first-payment-date-for-woocommerce-subscriptions' ),
				'description' => __( 'Set the date when the first payment of this subscription has to be done.', 'first-payment-date-for-woocommerce-subscriptions' ),
				'input_class' => array( 'hasDatepicker' ),
				'value' 	  => get_post_meta( $product->get_id(), '_fpdws_date_first_payment', true ),
				 'type' 		  => 'date',
			)
		 );
		 
		woocommerce_wp_select( array(
			'id'      => '_fpdws_date_first_payment_calculated',
			'label'   => __( 'First payment date', 'first-payment-date-for-woocommerce-subscriptions' ),
			'options' => array( 
				'first_day_next_month' => __( 'First day next month', 'first-payment-date-for-woocommerce-subscriptions' ), 
				'first_day_next_year' => __( 'First day next year', 'first-payment-date-for-woocommerce-subscriptions' ), 
			),
			'value'	  => get_post_meta( $product->get_id(), '_fpdws_date_first_payment_calculated', true ),
		) );
	
		do_action( 'fpdws_after_simple_subscription_product_options', $product );

		echo '</div>';
	}

	function update_first_date_payment_simple_subscription( $post_id ){
		if( isset( $_POST['_fpdws_date_first_payment_type'] ) )
			update_post_meta( $post_id, '_fpdws_date_first_payment_type', sanitize_text_field( $_POST['_fpdws_date_first_payment_type'] ) );
	
		if( isset( $_POST['_fpdws_date_first_payment'] ) )
			update_post_meta( $post_id, '_fpdws_date_first_payment', sanitize_text_field( $_POST['_fpdws_date_first_payment'] ) );
	
		if( isset( $_POST['_fpdws_date_first_payment_calculated'] ) )
			update_post_meta( $post_id, '_fpdws_date_first_payment_calculated', sanitize_text_field( $_POST['_fpdws_date_first_payment_calculated'] ) );
	}

	function set_first_date_payment_variable_subscription( $loop, $variation_data, $variation ) {
		echo '<div class="options_group form-row form-row-full show_if_variable-subscription">';
			woocommerce_wp_select( array(
				'id'      => '_fpdws_date_first_payment_type_' . $variation->ID,
				'name'      => '_fpdws_date_first_payment_type[' . $variation->ID . ']',
				'label'   => __( 'First payment date type', 'first-payment-date-for-woocommerce-subscriptions' ),
				'options' => array( 
					'none' => __( 'Use standard trial period', 'first-payment-date-for-woocommerce-subscriptions' ),
					'fixed' => __( 'Fixed date', 'first-payment-date-for-woocommerce-subscriptions' ),
					'calculated' => __( 'Calculated day', 'first-payment-date-for-woocommerce-subscriptions' ),
				),
				'value'	  => get_post_meta( $variation->ID, '_fpdws_date_first_payment_type', true ),
			) );
		
			woocommerce_wp_text_input(
				array(
					'id'          => '_fpdws_date_first_payment_' . $variation->ID,
					'name'          => '_fpdws_date_first_payment[' . $variation->ID . ']',
					'label'       => __( 'First payment date', 'first-payment-date-for-woocommerce-subscriptions' ),
					'desc_tip'    => true,
					'description' => __( 'Set the date when the first payment of this subscription has to be done.', 'first-payment-date-for-woocommerce-subscriptions' ),
					'type' 		  => 'date',
					'value' 	  => get_post_meta( $variation->ID, '_fpdws_date_first_payment', true )
			) );
			 
			woocommerce_wp_select( array(
				'id'      => '_fpdws_date_first_payment_calculated' . $variation->ID,
				'name'      => '_fpdws_date_first_payment_calculated[' . $variation->ID . ']',
				'label'   => __( 'First payment date', 'first-payment-date-for-woocommerce-subscriptions' ),
				'options' => array( 
					'first_day_next_month' => __( 'First day next month', 'first-payment-date-for-woocommerce-subscriptions' ), 
					'first_day_next_year' => __( 'First day next year', 'first-payment-date-for-woocommerce-subscriptions' ), 
				),
				'value'	  => get_post_meta( $variation->ID, '_fpdws_date_first_payment_calculated', true ),
			) );

			do_action( 'fpdws_after_variation_subscription_product_options', $loop, $variation_data, $variation );
	
		echo '</div>';	
		?>	
		<script>
		jQuery( document ).ready( function( $ ){
			function fpwds_hide_show<?php echo $variation->ID;?>(){
				var type = $( "#_fpdws_date_first_payment_type_<?php echo $variation->ID;?>" ).val();
				
				if( type == 'none' ){
					$( "._fpdws_date_first_payment_<?php echo $variation->ID;?>_field" ).hide();
					$( "._fpdws_date_first_payment_calculated<?php echo $variation->ID;?>_field" ).hide();
					$( "label[for='variable_subscription_trial_length[<?php echo $loop;?>]']" ).show();
					$( "input[name='variable_subscription_trial_length[<?php echo $loop;?>]']" ).show();
					$( "select[name='variable_subscription_trial_period[<?php echo $loop;?>]']" ).show();
				}
				else if( type == 'fixed' ){
					$( "._fpdws_date_first_payment_<?php echo $variation->ID;?>_field" ).show();
					$( "._fpdws_date_first_payment_calculated<?php echo $variation->ID;?>_field" ).hide();
					$( "label[for='variable_subscription_trial_length[<?php echo $loop;?>]']" ).hide();
					$( "input[name='variable_subscription_trial_length[<?php echo $loop;?>]']" ).hide();
					$( "select[name='variable_subscription_trial_period[<?php echo $loop;?>]']" ).hide();
					$( "input[name='variable_subscription_trial_length[<?php echo $loop;?>]']" ).val( 0 ).change();
					$( "select[name='variable_subscription_trial_period[<?php echo $loop;?>]']" ).val( 'day' ).change();
				}				
				else if( type == 'calculated' ){
					$( "._fpdws_date_first_payment_calculated<?php echo $variation->ID;?>_field" ).show();
					$( "._fpdws_date_first_payment_<?php echo $variation->ID;?>_field" ).hide();
					$( "label[for='variable_subscription_trial_length[<?php echo $loop;?>]']" ).hide();
					$( "input[name='variable_subscription_trial_length[<?php echo $loop;?>]']" ).hide();
					$( "select[name='variable_subscription_trial_period[<?php echo $loop;?>]']" ).hide();
					$( "input[name='variable_subscription_trial_length[<?php echo $loop;?>]']" ).val( 0 ).change();
					$( "select[name='variable_subscription_trial_period[<?php echo $loop;?>]']" ).val( 'day' ).change();
				}
			}
	
			$( "#_fpdws_date_first_payment_type_<?php echo $variation->ID;?>" ).change( function(){
				fpwds_hide_show<?php echo $variation->ID;?>();
			} );
	
			fpwds_hide_show<?php echo $variation->ID;?>();
		} );
		</script>
		<?php
	}
	
	function update_first_date_payment_variable_subscription( $post_id ){
		update_post_meta( $post_id, '_fpdws_date_first_payment_type', sanitize_text_field( $_POST['_fpdws_date_first_payment_type'][ $post_id ] ) );
		update_post_meta( $post_id, '_fpdws_date_first_payment', sanitize_text_field( $_POST['_fpdws_date_first_payment'][ $post_id ] ) );
		update_post_meta( $post_id, '_fpdws_date_first_payment_calculated', sanitize_text_field( $_POST['_fpdws_date_first_payment_calculated'][ $post_id ] ) );
	}
	
	function bulk_edit_actions() {
		global $post;

		if ( WC_Subscriptions_Product::is_subscription( $post->ID ) ) : ?>
			<optgroup label="<?php esc_attr_e( 'First date payment', 'first-payment-date-for-woocommerce-subscriptions' ); ?>">
				<option value="fpdws_set_fixed_first_payment_date"><?php esc_html_e( 'Set fixed first payment date', 'first-payment-date-for-woocommerce-subscriptions' ); ?></option>
			</optgroup>
		<?php endif;
	}

	function bulk_edit_variations( $bulk_action, $data, $variable_product_id, $variation_ids ){
		if( 'fpdws_set_fixed_first_payment_date' != $bulk_action )
			return;

		if( !isset( $data['value'] ) )
			return;

		$date = sanitize_text_field( $data['value'] );
		if( !preg_match( "/^[0-9]{4}-[0-9]{2}-[0-9]{2}$/", $date ) )
			return;

		$updated = 0;
		
		foreach ( $variation_ids as $variation_id ) {
			update_post_meta( $variation_id, '_fpdws_date_first_payment_type', 'fixed' );
			update_post_meta( $variation_id, '_fpdws_date_first_payment', $date );
			delete_post_meta( $variation_id, '_fpdws_date_first_payment_calculated' );
			update_post_meta( $variation_id, '_subscription_trial_length', 0 );
			delete_post_meta( $variation_id, '_subscription_trial_period', 'day' );

			$updated++;
		}

		echo intval( $updated );
		return;
	}

	function footer_scripts(){
		global $post;

		if( empty( $post ) || !is_a( $post, 'WP_Post' ) )
			return;
		?>
		<script>
		jQuery( document ).ready( function( $ ){
			function fpwds_hide_show(){
				var type = $( '#_fpdws_date_first_payment_type' ).val();
				
				if( type == 'none' ){
					$( '._fpdws_date_first_payment_field' ).hide();
					$( '._fpdws_date_first_payment_calculated_field' ).hide();
					$( '._subscription_trial_length_field' ).show();
				}
				else if( type == 'fixed' ){
					$( '._fpdws_date_first_payment_field' ).show();
					$( '._fpdws_date_first_payment_calculated_field' ).hide();
					$( '._subscription_trial_length_field' ).hide();
					$( '#_subscription_trial_length' ).val( 0 ).change();
					$( '#_subscription_trial_period' ).val( 'day' ).change();
				}				
				else if( type == 'calculated' ){
					$( '._fpdws_date_first_payment_calculated_field' ).show();
					$( '._fpdws_date_first_payment_field' ).hide();
					$( '._subscription_trial_length_field' ).hide();
					$( '#_subscription_trial_length' ).val( 0 ).change();
					$( '#_subscription_trial_period' ).val( 'day' ).change();					
				}				
			}
	
			$( '#_fpdws_date_first_payment_type' ).change( function(){
				fpwds_hide_show();
			} );
	
			fpwds_hide_show();

			$( 'select.variation_actions' ).on( 'fpdws_set_fixed_first_payment_date_ajax_data',
				function ( event, data ) {
					value = prompt( '<?php _e( 'Fixed first payment date (format: YYYY-MM-DD)', 'first-payment-date-for-woocommerce-subscriptions' ) ?>', '<?php echo date( 'Y-m-d' ); ?>' );

					if ( value != null ) {
						data.value = value;
					}

					$( '.wc_input_subscription_trial_length' ).each( function(){
						$( this ).val( 0 );
					} )

					return data;
				}
			);
		} );
		</script>
		<?php
	}

	function change_woocommerce_subscription_lengths( $subscription_locale_ranges, $subscription_period ){
		if( !empty( $subscription_period ) && $subscription_period != 'day' )
			return $subscription_locale_ranges;

		if( !is_admin() )
			return $subscription_locale_ranges;
		
		$subscription_range = range( 91, 730 );
		foreach ( $subscription_range as $number ) {
			$subscription_locale_ranges['day'][ $number ] = wcs_get_subscription_period_strings( $number, 'day' );
		}

		return $subscription_locale_ranges;
	}

	function show_payment_schedule_message_simple(){
		if( !function_exists( 'is_plugin_active' ) )
			include_once ABSPATH . 'wp-admin/includes/plugin.php';
			
		if( is_plugin_active( 'payment-schedule-for-woocommerce-subscriptions/payment-schedule-for-woocommerce-subscriptions.php' ) )
			return;
		?>
		<p class="form-field" style="padding-left: 12px !important;">
			<?php echo sprintf( __( '<strong><a href="%s">Payment Schedule for WooCommerce Subscriptions</a>, the premium version of this plugin</strong> allows you to indicate the <strong>end date</strong> of your subscriptions, view and display your subscription <strong>payment schedule</strong> and <strong>customize your WooCommerce Subscriptions messages</strong>.', 'first-payment-date-for-woocommerce-subscriptions' ), $this->premium_url ); ?>
		</p>
		<?php
	}

	function show_payment_schedule_message_variation(){
		if( !function_exists( 'is_plugin_active' ) )
			include_once ABSPATH . 'wp-admin/includes/plugin.php';
			
		if( is_plugin_active( 'payment-schedule-for-woocommerce-subscriptions/payment-schedule-for-woocommerce-subscriptions.php' ) )
			return;
		?>
		<p class="form-field">
			<?php echo sprintf( __( '<strong><a href="%s">Payment Schedule for WooCommerce Subscriptions</a>, the premium version of this plugin</strong> allows you to indicate the <strong>end date</strong> of your subscriptions, view and display your subscription <strong>payment schedule</strong> and <strong>customize your WooCommerce Subscriptions messages</strong>.', 'first-payment-date-for-woocommerce-subscriptions' ), $this->premium_url ); ?>
		</p>
		<?php
	}
}